/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "activeconnection_nm.h"

#include <string.h>

#include "activeconnection.h"
#include "connection.h"
#include "dbus_wrapper.h"
#include "port.h"

#include "network_nm.h"

#include "activeconnection_private.h"
#include "network_private.h"
#include "errors.h"

typedef struct ActiveConnectionPriv {
    DBusGProxy *proxy;
    char *objectpath;
    GHashTable *properties;
} ActiveConnectionPriv;

ActiveConnection *active_connection_from_objectpath(Network *network, const char *objectpath, LMIResult *res)
{
    ActiveConnection *activeConnection = NULL;
    ActiveConnectionPriv *priv = malloc(sizeof(ActiveConnectionPriv));
    if (priv == NULL) {
        error("Memory allocation failed");
        *res = LMI_ERROR_MEMORY;
        goto err;
    }
    priv->objectpath = NULL;
    activeConnection = active_connection_new();
    if (activeConnection == NULL) {
        error("Memory allocation failed");
        *res = LMI_ERROR_MEMORY;
        goto err;
    }
    activeConnection->ports = ports_new(0);
    activeConnection->priv = priv;
    if ((priv->objectpath = strdup(objectpath)) == NULL) {
        error("Memory allocation failed");
        *res = LMI_ERROR_MEMORY;
        goto err;
    }

    priv->proxy = dbus_g_proxy_new_for_name(network_priv_get_dbus_connection(network), NM_SERVICE_DBUS, objectpath, NM_DBUS_INTERFACE_ACTIVE_CONNECTION);
    if (priv->proxy == NULL) {
        error("Unable to create DBus proxy: %s %s NM_DBUS_INTERFACE_ACTIVE_CONNECTION", NM_SERVICE_DBUS, objectpath);
        *res = LMI_ERROR_BACKEND;
        goto err;
    }

    priv->properties = dbus_get_properties(priv->proxy, objectpath, NM_DBUS_INTERFACE_ACTIVE_CONNECTION);
    if (priv->properties == NULL) {
        error("Unable to get properties for object %s", objectpath);
        *res = LMI_ERROR_BACKEND;
        goto err;
    }
    GPtrArray *array = dbus_property_array(priv->properties, "Devices");
    char *devicepath;
    Port *port;
    for (guint i = 0; i < array->len; ++i) {
        devicepath = (char *) g_ptr_array_index(array, i);
        port = ports_find_by_uuid(network->ports, devicepath);
        if (port == NULL) {
            warn("No such port: %s", devicepath);
            continue;
        }
        if ((*res = ports_add(activeConnection->ports, port)) != LMI_SUCCESS) {
            goto err;
        }
    }
    const char *connectionpath = dbus_property_objectpath(priv->properties, "Connection");
    Connection *connection = connections_find_by_uuid(network->connections, connectionpath);
    if (connection != NULL) {
        activeConnection->connection = connection;
    } else {
        warn("No such connection: %s", connectionpath);
    }
    return activeConnection;
err:
    if (priv != NULL) {
        free(priv->objectpath);
    }
    free(priv);
    active_connection_free(activeConnection);
    return NULL;
}

void activeconnection_priv_free(void *priv)
{
    if (priv == NULL) {
        return;
    }
    ActiveConnectionPriv *p = priv;
    free(p->objectpath);
    g_hash_table_destroy(p->properties);
    g_object_unref(p->proxy);
    free(p);
}
